@@ -34,6 +34,12 @@ import time
 
 from ansible.module_utils.connection import Connection
 
+checkpoint_argument_spec_for_action_module = dict(
+    auto_publish_session=dict(type='bool'),
+    wait_for_task_timeout=dict(type='int', default=30),
+    version=dict(type='str')
+)
+
 checkpoint_argument_spec_for_objects = dict(
     auto_publish_session=dict(type='bool'),
     wait_for_task=dict(type='bool', default=True),
@@ -131,8 +137,13 @@ def wait_for_task(module, version, connection, task_id):
         completed_tasks = 0
         for task in response['tasks']:
             if task['status'] == 'failed':
-                module.fail_json(msg='Task {0} with task id {1} failed. Look at the logs for more details'
-                                 .format(task['task-name'], task['task-id']))
+                if 'comments' in task and task['comments']:
+                    module.fail_json(msg='Task {0} with task id {1} failed. Message: {2} - Look at the logs for more '
+                                         'details '
+                                     .format(task['task-name'], task['task-id'], task['comments']))
+                else:
+                    module.fail_json(msg='Task {0} with task id {1} failed. Look at the logs for more details'
+                                     .format(task['task-name'], task['task-id']))
             if task['status'] == 'in progress':
                 break
             completed_tasks += 1
@@ -211,7 +222,9 @@ def api_command(module, command):
             elif 'tasks' in response:
                 for task in response['tasks']:
                     if 'task-id' in task:
-                        wait_for_task(module, version, connection, task['task-id'])
+                        task_id = task['task-id']
+                        response[task_id] = wait_for_task(module, version, connection, task['task-id'])
+                del response['tasks']
 
         result[command] = response
     else:
@@ -497,3 +510,21 @@ def install_policy(connection, policy_package, targets):
                'targets': targets}
 
     connection.send_request('/web_api/install-policy', payload)
+
+
+def prepare_rule_params_for_execute_module(rule, module_args, position):
+    rule['layer'] = module_args['layer']
+    if 'details_level' in module_args.keys():
+        rule['details_level'] = module_args['details_level']
+    if 'state' not in rule.keys() or ('state' in rule.keys() and rule['state'] != 'absent'):
+        rule['position'] = position
+        position = position + 1
+
+    return rule, position
+
+
+def check_if_to_publish_for_action(result, module_args):
+    to_publish = ('auto_publish_session' in module_args.keys() and module_args['auto_publish_session']) and \
+                 ('changed' in result.keys() and result['changed'] is True) and ('failed' not in result.keys() or
+                                                                                 result['failed'] is False)
+    return to_publish
