@@ -19,8 +19,6 @@ author:
   - Fernando Jose Pando (@nand0p)
   - Nadir Lloret (@nadirollo)
   - Dennis Podkovyrin (@sbj-ss)
-requirements:
-  - boto3
 options:
   state:
     description:
@@ -88,7 +86,7 @@ options:
       - Defaults to C(false).
   tags:
     description:
-      - Tag dict to apply to the queue (requires botocore 1.5.40 or above).
+      - Tag dict to apply to the queue.
       - To remove all tags set I(tags={}) and I(purge_tags=true).
     type: dict
   purge_tags:
@@ -121,7 +119,7 @@ delay_seconds:
 kms_master_key_id:
     description: The ID of an AWS-managed customer master key (CMK) for Amazon SQS or a custom CMK.
     type: str
-    returned: always
+    returned: if value exists
     sample: alias/MyAlias
 kms_data_key_reuse_period_seconds:
     description: The length of time, in seconds, for which Amazon SQS can reuse a data key to encrypt or decrypt messages before calling AWS KMS again.
@@ -292,6 +290,7 @@ def describe_queue(client, queue_url):
 
 def create_or_update_sqs_queue(client, module):
     is_fifo = (module.params.get('queue_type') == 'fifo')
+    kms_master_key_id = module.params.get('kms_master_key_id')
     queue_name = get_queue_name(module, is_fifo)
     result = dict(
         name=queue_name,
@@ -302,8 +301,14 @@ def create_or_update_sqs_queue(client, module):
     queue_url = get_queue_url(client, queue_name)
     result['queue_url'] = queue_url
 
+    # Create a dict() to hold attributes that will be passed to boto3
+    create_attributes = {}
+
     if not queue_url:
-        create_attributes = {'FifoQueue': 'true'} if is_fifo else {}
+        if is_fifo:
+            create_attributes['FifoQueue'] = "True"
+        if kms_master_key_id:
+            create_attributes['KmsMasterKeyId'] = kms_master_key_id
         result['changed'] = True
         if module.check_mode:
             return result
@@ -375,9 +380,9 @@ def update_sqs_queue(module, client, queue_url):
 
         if isinstance(new_value, bool):
             new_value = str(new_value).lower()
-            existing_value = str(existing_value).lower()
+            value = str(value).lower()
 
-        if new_value == value:
+        if str(new_value) == str(value):
             continue
 
         # Boto3 expects strings
@@ -387,7 +392,7 @@ def update_sqs_queue(module, client, queue_url):
     if changed and not check_mode:
         client.set_queue_attributes(QueueUrl=queue_url, Attributes=attributes_to_set, aws_retry=True)
 
-    return changed, existing_attributes.get('queue_arn'),
+    return changed, existing_attributes.get('queue_arn')
 
 
 def delete_sqs_queue(client, module):
@@ -450,7 +455,7 @@ def main():
         redrive_policy=dict(type='dict'),
         visibility_timeout=dict(type='int', aliases=['default_visibility_timeout']),
         kms_master_key_id=dict(type='str'),
-        kms_data_key_reuse_period_seconds=dict(type='int', aliases=['kms_data_key_reuse_period']),
+        kms_data_key_reuse_period_seconds=dict(type='int', aliases=['kms_data_key_reuse_period'], no_log=False),
         content_based_deduplication=dict(type='bool'),
         tags=dict(type='dict'),
         purge_tags=dict(type='bool', default=False),
