@@ -163,67 +163,71 @@ extends_documentation_fragment:
 
 
 EXAMPLES = r"""
-- name: Create a new droplet
+- name: Create a new Droplet
   community.digitalocean.digital_ocean_droplet:
     state: present
+    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
     name: mydroplet
-    oauth_token: XXX
-    size: 2gb
-    region: sfo1
-    image: ubuntu-18-04-x64
+    size: s-1vcpu-1gb
+    region: sfo3
+    image: ubuntu-20-04-x64
     wait_timeout: 500
     ssh_keys: [ .... ]
   register: my_droplet
 
-- debug:
-    msg: "ID is {{ my_droplet.data.droplet.id }}, IP is {{ my_droplet.data.ip_address }}"
+- name: Show Droplet info
+  ansible.builtin.debug:
+    msg: |
+      Droplet ID is {{ my_droplet.data.droplet.id }}
+      First Public IPv4 is {{ (my_droplet.data.droplet.networks.v4 | selectattr('type', 'equalto', 'public')).0.ip_address | default('<none>', true) }}
+      First Private IPv4 is {{ (my_droplet.data.droplet.networks.v4 | selectattr('type', 'equalto', 'private')).0.ip_address | default('<none>', true) }}
 
-- name: Create a new droplet (and assign to Project "test")
+- name: Create a new Droplet (and assign to Project "test")
   community.digitalocean.digital_ocean_droplet:
     state: present
+    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
     name: mydroplet
-    oauth_token: XXX
-    size: 2gb
-    region: sfo1
-    image: ubuntu-16-04-x64
+    size: s-1vcpu-1gb
+    region: sfo3
+    image: ubuntu-20-04-x64
     wait_timeout: 500
     ssh_keys: [ .... ]
     project: test
   register: my_droplet
 
-- name: Ensure a droplet is present
+- name: Ensure a Droplet is present
   community.digitalocean.digital_ocean_droplet:
     state: present
+    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
     id: 123
     name: mydroplet
-    oauth_token: XXX
-    size: 2gb
-    region: sfo1
-    image: ubuntu-18-04-x64
+    size: s-1vcpu-1gb
+    region: sfo3
+    image: ubuntu-20-04-x64
     wait_timeout: 500
 
-- name: Ensure a droplet is present and has firewall rules applied
+- name: Ensure a Droplet is present and has firewall rules applied
   community.digitalocean.digital_ocean_droplet:
     state: present
+    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
     id: 123
     name: mydroplet
-    oauth_token: XXX
-    size: 2gb
-    region: sfo1
-    image: ubuntu-18-04-x64
+    size: s-1vcpu-1gb
+    region: sfo3
+    image: ubuntu-20-04-x64
     firewall: ['myfirewall', 'anotherfirewall']
     wait_timeout: 500
 
-- name: Ensure a droplet is present with SSH keys installed
+- name: Ensure a Droplet is present with SSH keys installed
   community.digitalocean.digital_ocean_droplet:
     state: present
+    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
     id: 123
     name: mydroplet
-    oauth_token: XXX
-    size: 2gb
-    region: sfo1
+    size: s-1vcpu-1gb
+    region: sfo3
     ssh_keys: ['1534404', '1784768']
-    image: ubuntu-18-04-x64
+    image: ubuntu-20-04-x64
     wait_timeout: 500
 """
 
@@ -729,8 +733,17 @@ class DODroplet(object):
         status_code = response.status_code
         message = json_data.get("message", "no error message")
         droplet = json_data.get("droplet", None)
-        droplet_id = droplet.get("id", None)
 
+        # Ensure that the Droplet is created
+        if status_code != 202:
+            self.module.fail_json(
+                changed=False,
+                msg=DODroplet.failure_message["failed_to"].format(
+                    "create", "Droplet", status_code, message
+                ),
+            )
+
+        droplet_id = droplet.get("id", None)
         if droplet is None or droplet_id is None:
             self.module.fail_json(
                 changed=False,
@@ -757,7 +770,9 @@ class DODroplet(object):
         # Get updated Droplet data (fallback to current data)
         if self.wait:
             json_data = self.get_droplet()
-            droplet = json_data.get("droplet", droplet)
+            # Without unique_name json_data is None
+            if json_data:
+                droplet = json_data.get("droplet", droplet)
 
         project_name = self.module.params.get("project")
         if project_name:  # empty string is the default project, skip project assignment
