@@ -10,20 +10,23 @@ import os
 import platform
 import re
 import sys
+import traceback
 from datetime import timedelta
-from distutils.version import LooseVersion
-
 
 from ansible.module_utils.basic import AnsibleModule, env_fallback, missing_required_lib
+from ansible.module_utils.common.collections import is_sequence
 from ansible.module_utils.common._collections_compat import Mapping, Sequence
 from ansible.module_utils.six import string_types
 from ansible.module_utils.six.moves.urllib.parse import urlparse
 from ansible.module_utils.parsing.convert_bool import BOOLEANS_TRUE, BOOLEANS_FALSE
 
+from ansible_collections.community.docker.plugins.module_utils.version import LooseVersion
+
 HAS_DOCKER_PY = True
 HAS_DOCKER_PY_2 = False
 HAS_DOCKER_PY_3 = False
 HAS_DOCKER_ERROR = None
+HAS_DOCKER_TRACEBACK = None
 
 try:
     from requests.exceptions import SSLError
@@ -43,10 +46,11 @@ try:
 
 except ImportError as exc:
     HAS_DOCKER_ERROR = str(exc)
+    HAS_DOCKER_TRACEBACK = traceback.format_exc()
     HAS_DOCKER_PY = False
 
 
-# The next 2 imports ``docker.models`` and ``docker.ssladapter`` are used
+# The next two imports ``docker.models`` and ``docker.ssladapter`` are used
 # to ensure the user does not have both ``docker`` and ``docker-py`` modules
 # installed, as they utilize the same namespace are are incompatible
 try:
@@ -77,13 +81,13 @@ except ImportError:
 DEFAULT_DOCKER_HOST = 'unix://var/run/docker.sock'
 DEFAULT_TLS = False
 DEFAULT_TLS_VERIFY = False
-DEFAULT_TLS_HOSTNAME = 'localhost'
+DEFAULT_TLS_HOSTNAME = 'localhost'  # deprecated
 MIN_DOCKER_VERSION = "1.8.0"
 DEFAULT_TIMEOUT_SECONDS = 60
 
 DOCKER_COMMON_ARGS = dict(
     docker_host=dict(type='str', default=DEFAULT_DOCKER_HOST, fallback=(env_fallback, ['DOCKER_HOST']), aliases=['docker_url']),
-    tls_hostname=dict(type='str', default=DEFAULT_TLS_HOSTNAME, fallback=(env_fallback, ['DOCKER_TLS_HOSTNAME'])),
+    tls_hostname=dict(type='str', fallback=(env_fallback, ['DOCKER_TLS_HOSTNAME'])),
     api_version=dict(type='str', default='auto', fallback=(env_fallback, ['DOCKER_API_VERSION']), aliases=['docker_api_version']),
     timeout=dict(type='int', default=DEFAULT_TIMEOUT_SECONDS, fallback=(env_fallback, ['DOCKER_TIMEOUT'])),
     ca_cert=dict(type='path', aliases=['tls_ca_cert', 'cacert_path']),
@@ -96,6 +100,12 @@ DOCKER_COMMON_ARGS = dict(
     debug=dict(type='bool', default=False)
 )
 
+DOCKER_COMMON_ARGS_VARS = dict([
+    [option_name, 'ansible_docker_%s' % option_name]
+    for option_name in DOCKER_COMMON_ARGS
+    if option_name != 'debug'
+])
+
 DOCKER_MUTUALLY_EXCLUSIVE = []
 
 DOCKER_REQUIRED_TOGETHER = [
@@ -103,7 +113,6 @@ DOCKER_REQUIRED_TOGETHER = [
 ]
 
 DEFAULT_DOCKER_REGISTRY = 'https://index.docker.io/v1/'
-EMAIL_REGEX = r'[^@]+@[^@]+\.[^@]+'
 BYTE_SUFFIXES = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
 
 
@@ -172,8 +181,18 @@ class DockerBaseClass(object):
         #         log_file.write(msg + u'\n')
 
 
-def update_tls_hostname(result):
+def update_tls_hostname(result, old_behavior=False, deprecate_function=None, uses_tls=True):
     if result['tls_hostname'] is None:
+        if old_behavior:
+            result['tls_hostname'] = DEFAULT_TLS_HOSTNAME
+            if uses_tls and deprecate_function is not None:
+                deprecate_function(
+                    'The default value "localhost" for tls_hostname is deprecated and will be removed in community.docker 2.0.0.'
+                    ' From then on, docker_host will be used to compute tls_hostname. If you want to keep using "localhost",'
+                    ' please set that value explicitly.',
+                    version='2.0.0', collection_name='community.docker')
+            return
+
         # get default machine name from the url
         parsed_url = urlparse(result['docker_host'])
         if ':' in parsed_url.netloc:
@@ -190,8 +209,12 @@ def _get_tls_config(fail_function, **kwargs):
         fail_function("TLS config error: %s" % exc)
 
 
+def is_using_tls(auth):
+    return auth['tls_verify'] or auth['tls']
+
+
 def get_connect_params(auth, fail_function):
-    if auth['tls'] or auth['tls_verify']:
+    if is_using_tls(auth):
         auth['docker_host'] = auth['docker_host'].replace('tcp://', 'https://')
 
     result = dict(
@@ -258,12 +281,16 @@ class AnsibleDockerClientBase(Client):
 
         if not HAS_DOCKER_PY:
             if NEEDS_DOCKER_PY2:
-                msg = missing_required_lib("Docker SDK for Python: docker")
-                msg = msg + ", for example via `pip install docker`. The error was: %s"
+                msg = missing_required_lib("Docker SDK for Python: docker above 5.0.0 (Python >= 3.6) or "
+                                           "docker before 5.0.0 (Python 2.7)")
+                msg = msg + ", for example via `pip install docker` (Python >= 3.6) or " \
+                    + "`pip install docker==4.4.4` (Python 2.7). The error was: %s"
             else:
-                msg = missing_required_lib("Docker SDK for Python: docker (Python >= 2.7) or docker-py (Python 2.6)")
-                msg = msg + ", for example via `pip install docker` or `pip install docker-py` (Python 2.6). The error was: %s"
-            self.fail(msg % HAS_DOCKER_ERROR)
+                msg = missing_required_lib("Docker SDK for Python: docker above 5.0.0 (Python >= 3.6) or "
+                                           "docker before 5.0.0 (Python 2.7) or docker-py (Python 2.6)")
+                msg = msg + ", for example via `pip install docker` (Python >= 3.6) or `pip install docker==4.4.4` (Python 2.7) " \
+                    + "or `pip install docker-py` (Python 2.6). The error was: %s"
+            self.fail(msg % HAS_DOCKER_ERROR, exception=HAS_DOCKER_TRACEBACK)
 
         if self.docker_py_version < LooseVersion(min_docker_version):
             msg = "Error: Docker SDK for Python version is %s (%s's Python %s). Minimum version required is %s."
@@ -306,6 +333,9 @@ class AnsibleDockerClientBase(Client):
     def fail(self, msg, **kwargs):
         pass
 
+    def deprecate(self, msg, version=None, date=None, collection_name=None):
+        pass
+
     @staticmethod
     def _get_value(param_name, param_value, env_variable, default_value):
         if param_value is not None:
@@ -356,7 +386,7 @@ class AnsibleDockerClientBase(Client):
             docker_host=self._get_value('docker_host', params['docker_host'], 'DOCKER_HOST',
                                         DEFAULT_DOCKER_HOST),
             tls_hostname=self._get_value('tls_hostname', params['tls_hostname'],
-                                         'DOCKER_TLS_HOSTNAME', DEFAULT_TLS_HOSTNAME),
+                                         'DOCKER_TLS_HOSTNAME', None),
             api_version=self._get_value('api_version', params['api_version'], 'DOCKER_API_VERSION',
                                         'auto'),
             cacert_path=self._get_value('cacert_path', params['ca_cert'], 'DOCKER_CERT_PATH', None),
@@ -371,7 +401,10 @@ class AnsibleDockerClientBase(Client):
             use_ssh_client=self._get_value('use_ssh_client', params['use_ssh_client'], None, False),
         )
 
-        update_tls_hostname(result)
+        def depr(*args, **kwargs):
+            self.deprecate(*args, **kwargs)
+
+        update_tls_hostname(result, old_behavior=True, deprecate_function=depr, uses_tls=is_using_tls(result))
 
         return result
 
@@ -653,6 +686,9 @@ class AnsibleDockerClient(AnsibleDockerClientBase):
         self.fail_results.update(kwargs)
         self.module.fail_json(msg=msg, **sanitize_result(self.fail_results))
 
+    def deprecate(self, msg, version=None, date=None, collection_name=None):
+        self.module.deprecate(msg, version=version, date=date, collection_name=collection_name)
+
     def _get_params(self):
         return self.module.params
 
@@ -894,23 +930,27 @@ class DifferenceTracker(object):
         return result
 
 
-def clean_dict_booleans_for_docker_api(data):
+def clean_dict_booleans_for_docker_api(data, allow_sequences=False):
     '''
     Go doesn't like Python booleans 'True' or 'False', while Ansible is just
     fine with them in YAML. As such, they need to be converted in cases where
     we pass dictionaries to the Docker API (e.g. docker_network's
-    driver_options and docker_prune's filters).
+    driver_options and docker_prune's filters). When `allow_sequences=True`
+    YAML sequences (lists, tuples) are converted to [str] instead of str([...])
+    which is the expected format of filters which accept lists such as labels.
     '''
+    def sanitize(value):
+        if value is True:
+            return 'true'
+        elif value is False:
+            return 'false'
+        else:
+            return str(value)
+
     result = dict()
     if data is not None:
         for k, v in data.items():
-            if v is True:
-                v = 'true'
-            elif v is False:
-                v = 'false'
-            else:
-                v = str(v)
-            result[str(k)] = v
+            result[str(k)] = [sanitize(e) for e in v] if allow_sequences and is_sequence(v) else sanitize(v)
     return result
 
 
