@@ -17,8 +17,7 @@ short_description: Manage multi-container Docker applications with Docker Compos
 author: "Chris Houseknecht (@chouseknecht)"
 
 description:
-  - Uses Docker Compose to start, shutdown and scale services.
-  - Works with compose versions 1 and 2.
+  - Uses Docker Compose to start, shutdown and scale services. B(This module requires docker-compose < 2.0.0.)
   - Configuration can be read from a C(docker-compose.yml) or C(docker-compose.yaml) file or inline using the I(definition) option.
   - See the examples for more details.
   - Supports check mode.
@@ -36,12 +35,30 @@ options:
       - Provide a project name. If not provided, the project name is taken from the basename of I(project_src).
       - Required when I(definition) is provided.
     type: str
+  env_file:
+    description:
+      - By default environment files are loaded from a C(.env) file located directly under the I(project_src) directory.
+      - I(env_file) can be used to specify the path of a custom environment file instead.
+      - The path is relative to the I(project_src) directory.
+      - Requires C(docker-compose) version 1.25.0 or greater.
+      - "Note: C(docker-compose) versions C(<=1.28) load the env file from the current working directory of the
+          C(docker-compose) command rather than I(project_src)."
+    type: path
+    version_added: 1.9.0
   files:
     description:
       - List of Compose file names relative to I(project_src). Overrides C(docker-compose.yml) or C(docker-compose.yaml).
       - Files are loaded and merged in the order given.
     type: list
     elements: path
+  profiles:
+    description:
+      - List of profiles to enable when starting services.
+      - Equivalent to C(docker-compose --profile).
+      - Requires C(docker-compose) version 1.28.0 or greater.
+    type: list
+    elements: str
+    version_added: 1.8.0
   state:
     description:
       - Desired state of the project.
@@ -128,6 +145,8 @@ options:
     description:
       - Use with I(state) C(present) to stop all containers defined in the Compose file.
       - If I(services) is defined, only the containers listed there will be stopped.
+      - Requires C(docker-compose) version 1.17.0 or greater for full support. For older versions, the services will
+        first be started and then stopped when the service is supposed to be created as stopped.
     type: bool
     default: no
   restarted:
@@ -144,8 +163,10 @@ options:
   timeout:
     description:
       - Timeout in seconds for container shutdown when attached or when containers are already running.
+      - By default C(compose) will use a C(10s) timeout unless C(default_grace_period) is defined for a
+        particular service in the I(project_src).
     type: int
-    default: 10
+    default: null
   use_ssh_client:
     description:
       - Currently ignored for this module, but might suddenly be supported later on.
@@ -157,7 +178,7 @@ extends_documentation_fragment:
 
 requirements:
   - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 1.8.0 (use L(docker-py,https://pypi.org/project/docker-py/) for Python 2.6)"
-  - "docker-compose >= 1.7.0"
+  - "docker-compose >= 1.7.0, < 2.0.0"
   - "Docker API >= 1.20"
   - "PyYAML >= 3.11"
 '''
@@ -455,7 +476,8 @@ import sys
 import tempfile
 import traceback
 from contextlib import contextmanager
-from distutils.version import LooseVersion
+
+from ansible_collections.community.docker.plugins.module_utils.version import LooseVersion
 
 try:
     import yaml
@@ -476,16 +498,15 @@ try:
     from compose.cli.command import project_from_options
     from compose.service import NoSuchImageError
     from compose.cli.main import convergence_strategy_from_opts, build_action_from_opts, image_type_from_opt
-    from compose.const import DEFAULT_TIMEOUT, LABEL_SERVICE, LABEL_PROJECT, LABEL_ONE_OFF
+    from compose.const import LABEL_SERVICE, LABEL_PROJECT, LABEL_ONE_OFF
     HAS_COMPOSE = True
     HAS_COMPOSE_EXC = None
     MINIMUM_COMPOSE_VERSION = '1.7.0'
 except ImportError as dummy:
     HAS_COMPOSE = False
     HAS_COMPOSE_EXC = traceback.format_exc()
-    DEFAULT_TIMEOUT = 10
 
-from ansible.module_utils._text import to_native
+from ansible.module_utils.common.text.converters import to_native
 
 from ansible_collections.community.docker.plugins.module_utils.common import (
     AnsibleDockerClient,
@@ -629,9 +650,15 @@ class ContainerManager(DockerBaseClass):
         if self.project_name:
             self.options[u'--project-name'] = self.project_name
 
+        if self.env_file:
+            self.options[u'--env-file'] = self.env_file
+
         if self.files:
             self.options[u'--file'] = self.files
 
+        if self.profiles:
+            self.options[u'--profile'] = self.profiles
+
         if not HAS_COMPOSE:
             self.client.fail("Unable to load docker-compose. Try `pip install docker-compose`. Error: %s" %
                              to_native(HAS_COMPOSE_EXC))
@@ -730,12 +757,12 @@ class ContainerManager(DockerBaseClass):
 
         if self.pull:
             pull_output = self.cmd_pull()
-            result['changed'] = pull_output['changed']
+            result['changed'] |= pull_output['changed']
             result['actions'] += pull_output['actions']
 
         if self.build:
             build_output = self.cmd_build()
-            result['changed'] = build_output['changed']
+            result['changed'] |= build_output['changed']
             result['actions'] += build_output['actions']
 
         if self.remove_orphans:
@@ -760,6 +787,12 @@ class ContainerManager(DockerBaseClass):
         for service in self.project.services:
             if not service_names or service.name in service_names:
                 plan = service.convergence_plan(strategy=converge)
+                if plan.action == 'start' and self.stopped:
+                    # In case the only action is starting, and the user requested
+                    # that the service should be stopped, ignore this service.
+                    continue
+                if not self._service_profile_enabled(service):
+                    continue
                 if plan.action != 'noop':
                     result['changed'] = True
                     result_action = dict(service=service.name)
@@ -779,15 +812,25 @@ class ContainerManager(DockerBaseClass):
                     with stderr_redirector(err_redir_name):
                         do_build = build_action_from_opts(up_options)
                         self.log('Setting do_build to %s' % do_build)
-                        self.project.up(
-                            service_names=service_names,
-                            start_deps=start_deps,
-                            strategy=converge,
-                            do_build=do_build,
-                            detached=detached,
-                            remove_orphans=self.remove_orphans,
-                            timeout=self.timeout,
-                            start=not self.stopped)
+                        up_kwargs = {
+                            'service_names': service_names,
+                            'start_deps': start_deps,
+                            'strategy': converge,
+                            'do_build': do_build,
+                            'detached': detached,
+                            'remove_orphans': self.remove_orphans,
+                            'timeout': self.timeout,
+                        }
+
+                        if LooseVersion(compose_version) >= LooseVersion('1.17.0'):
+                            up_kwargs['start'] = not self.stopped
+                        elif self.stopped:
+                            self.client.module.warn(
+                                "The 'stopped' option requires docker-compose version >= 1.17.0. " +
+                                "This task was run with docker-compose version %s." % compose_version
+                            )
+
+                        self.project.up(**up_kwargs)
             except Exception as exc:
                 fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                                msg_format="Error starting project %s")
@@ -802,12 +845,12 @@ class ContainerManager(DockerBaseClass):
 
         if self.restarted:
             restart_output = self.cmd_restart(service_names)
-            result['changed'] = restart_output['changed']
+            result['changed'] |= restart_output['changed']
             result['actions'] += restart_output['actions']
 
         if self.scale:
             scale_output = self.cmd_scale()
-            result['changed'] = scale_output['changed']
+            result['changed'] |= scale_output['changed']
             result['actions'] += scale_output['actions']
 
         for service in self.project.services:
@@ -970,6 +1013,14 @@ class ContainerManager(DockerBaseClass):
                         ))
         return result
 
+    def _service_profile_enabled(self, service):
+        """Returns `True` if the service has no profiles defined or has a profile which is among
+           the profiles passed to the `docker compose up` command. Otherwise returns `False`.
+        """
+        if LooseVersion(compose_version) < LooseVersion('1.28.0'):
+            return True
+        return service.enabled_for_profiles(self.profiles or [])
+
     def cmd_down(self):
         result = dict(
             changed=False,
@@ -1109,7 +1160,9 @@ def main():
     argument_spec = dict(
         project_src=dict(type='path'),
         project_name=dict(type='str',),
+        env_file=dict(type='path'),
         files=dict(type='list', elements='path'),
+        profiles=dict(type='list', elements='str'),
         state=dict(type='str', default='present', choices=['absent', 'present']),
         definition=dict(type='dict'),
         hostname_check=dict(type='bool', default=False),
@@ -1126,7 +1179,7 @@ def main():
         pull=dict(type='bool', default=False),
         nocache=dict(type='bool', default=False),
         debug=dict(type='bool', default=False),
-        timeout=dict(type='int', default=DEFAULT_TIMEOUT)
+        timeout=dict(type='int')
     )
 
     mutually_exclusive = [
