@@ -39,6 +39,7 @@ options:
     - sni-thruk-datasource
     - camptocamp-prometheus-alertmanager-datasource
     - loki
+    - redis-datasource
     type: str
   ds_url:
     description:
@@ -455,10 +456,12 @@ import json
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.six.moves.urllib.parse import quote
 from ansible.module_utils.urls import fetch_url, url_argument_spec, basic_auth_header
-import ansible_collections.community.grafana.plugins.module_utils.base as base
+from ansible_collections.community.grafana.plugins.module_utils import base
 
 
 def compare_datasources(new, current, compareSecureData=True):
+    if 'uid' in current:
+        del current['uid']
     del current['typeLogoUrl']
     del current['id']
     if 'version' in current:
@@ -532,6 +535,7 @@ def get_datasource_payload(data):
         json_data['tlsAuth'] = False
         json_data['tlsAuthWithCACert'] = False
         if data.get('tls_ca_cert'):
+            json_data['tlsAuthWithCACert'] = True
             secure_json_data['tlsCACert'] = data['tls_ca_cert']
 
     if data.get('tls_skip_verify'):
@@ -626,11 +630,11 @@ class GrafanaInterface(object):
     def datasource_by_name(self, name):
         datasource_exists = False
         ds = {}
-        url = "/api/datasources/name/%s" % quote(name)
+        url = "/api/datasources/name/%s" % quote(name, safe='')
         return self._send_request(url, headers=self.headers, method='GET')
 
     def delete_datasource(self, name):
-        url = "/api/datasources/name/%s" % quote(name)
+        url = "/api/datasources/name/%s" % quote(name, safe='')
         self._send_request(url, headers=self.headers, method='DELETE')
 
     def update_datasource(self, ds_id, data):
@@ -659,6 +663,7 @@ def main():
                               'alexanderzobnin-zabbix-datasource',
                               'camptocamp-prometheus-alertmanager-datasource',
                               'sni-thruk-datasource',
+                              'redis-datasource',
                               'loki'], required=True),
         ds_url=dict(required=True, type='str'),
         access=dict(default='proxy', choices=['proxy', 'direct']),
