@@ -291,6 +291,73 @@ options:
     default: False
     aliases:
       - restart
+  generate_systemd:
+    description:
+      - Generate systemd unit file for container.
+    type: dict
+    default: {}
+    suboptions:
+      path:
+        description:
+          - Specify a path to the directory where unit files will be generated.
+            Required for this option. If it doesn't exist, the directory will be created.
+        type: str
+        required: false
+      restart_policy:
+        description:
+          - Specify a restart policy for the service.  The restart-policy must be one of
+            "no", "on-success", "on-failure", "on-abnormal", "on-watchdog", "on-abort", or "always".
+            The default policy is "on-failure".
+        type: str
+        required: false
+        choices:
+            - 'no'
+            - 'on-success'
+            - 'on-failure'
+            - 'on-abnormal'
+            - 'on-watchdog'
+            - 'on-abort'
+            - 'always'
+      time:
+        description:
+          - Override the default stop timeout for the container with the given value.
+        type: int
+        required: false
+      no_header:
+        description:
+          - Do not generate the header including meta data such as the Podman version and the timestamp.
+            From podman version 3.1.0.
+        type: bool
+        default: false
+      names:
+        description:
+          - Use names of the containers for the start, stop, and description in the unit file.
+            Default is true.
+        type: bool
+        default: true
+      container_prefix:
+        description:
+          - Set the systemd unit name prefix for containers. The default is "container".
+        type: str
+        required: false
+      pod_prefix:
+        description:
+          - Set the systemd unit name prefix for pods. The default is "pod".
+        type: str
+        required: false
+      separator:
+        description:
+          - Set the systemd unit name separator between the name/id of a
+            container/pod and the prefix. The default is "-" (dash).
+        type: str
+        required: false
+      new:
+        description:
+          - Create containers and pods when the unit is started instead of
+            expecting them to exist. The default is "false".
+            Refer to podman-generate-systemd(1) for more information.
+        type: bool
+        default: false
   gidmap:
     description:
       - Run the container in a new user namespace using the supplied mapping.
@@ -486,7 +553,10 @@ options:
       - Attach a filesystem mount to the container. bind or tmpfs
         For example mount
         "type=bind,source=/path/on/host,destination=/path/in/container"
-    type: str
+    type: list
+    elements: str
+    aliases:
+      - mounts
   network:
     description:
       - Set the Network mode for the container
@@ -503,6 +573,13 @@ options:
     aliases:
       - net
       - network_mode
+  network_aliases:
+    description:
+      - Add network-scoped alias for the container.
+        A container will only have access to aliases on the first network that it joins.
+        This is a limitation that will be removed in a later release.
+    type: list
+    elements: str
   no_hosts:
     description:
       - Do not create /etc/hosts for the container
@@ -595,6 +672,11 @@ options:
       - If true, the first argument refers to an exploded container on the file
         system. The default is false.
     type: bool
+  secrets:
+    description:
+      - Add the named secrets into the container at /run/secrets.
+    type: list
+    elements: str
   security_opt:
     description:
       - Security Options. For example security_opt "seccomp=unconfined"
@@ -724,7 +806,7 @@ EXAMPLES = r"""
     volume:
       - /tmp/data
 
-- name: Re-create a redis container
+- name: Re-create a redis container with systemd service file generated in /tmp/
   containers.podman.podman_container:
     name: myredis
     image: redis
@@ -735,6 +817,12 @@ EXAMPLES = r"""
       - 6379
     volumes_from:
       - mydata
+    generate_systemd:
+      path: /tmp/
+      restart_policy: always
+      time: 120
+      names: true
+      container_prefix: ainer
 
 - name: Restart a container
   containers.podman.podman_container:
