@@ -47,7 +47,7 @@ try:
 except ImportError:
     ansible_version = 'unknown'
 
-COLLECTION_VERSION = "21.9.0"
+COLLECTION_VERSION = "21.14.1"
 CLIENT_APP_VERSION = "%s/" + COLLECTION_VERSION
 IMPORT_EXCEPTION = None
 
@@ -167,7 +167,10 @@ def get_feature(module, feature_name):
         sanitize_code_points=[8],               # unicode values, 8 is backspace
         show_modified=True,
         always_wrap_zapi=True,                  # for better error reporting
-        flexcache_delete_return_timeout=5       # ONTAP bug if too big?
+        flexcache_delete_return_timeout=5,      # ONTAP bug if too big?
+        # for SVM, whch protocols can be allowed
+        svm_allowable_protocols_rest=['cifs', 'fcp', 'iscsi', 'nvme', 'nfs'],
+        svm_allowable_protocols_zapi=['cifs', 'fcp', 'iscsi', 'nvme', 'nfs', 'ndmp', 'http'],
     )
 
     if module.params['feature_flags'] is not None and feature_name in module.params['feature_flags']:
@@ -184,8 +187,7 @@ def create_sf_connection(module, port=None):
 
     if HAS_SF_SDK and hostname and username and password:
         try:
-            return_val = ElementFactory.create(hostname, username, password, port=port)
-            return return_val
+            return ElementFactory.create(hostname, username, password, port=port)
         except Exception:
             raise Exception("Unable to create SF connection")
     module.fail_json(msg="the python SolidFire SDK module is required")
@@ -239,13 +241,30 @@ def setup_host_options_from_module_params(host_options, module, keys):
     elif use_certificate:
         exclude_keys = password_keys
     else:
-        exclude_keys = list()
+        exclude_keys = []
     for key in keys:
         if host_options.get(key) is None and key not in exclude_keys:
             # use same value as source if no value is given for dest
             host_options[key] = module.params[key]
 
 
+def set_zapi_port_and_transport(server, https, port, validate_certs):
+    # default is HTTP
+    if https:
+        if port is None:
+            port = 443
+        transport_type = 'HTTPS'
+        # HACK to bypass certificate verification
+        if validate_certs is False and not os.environ.get('PYTHONHTTPSVERIFY', '') and getattr(ssl, '_create_unverified_context', None):
+            ssl._create_default_https_context = ssl._create_unverified_context
+    else:
+        if port is None:
+            port = 80
+        transport_type = 'HTTP'
+    server.set_transport_type(transport_type)
+    server.set_port(port)
+
+
 def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False, host_options=None):
     if host_options is None:
         host_options = module.params
@@ -258,79 +277,66 @@ def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False, host_options=None
     validate_certs = host_options.get('validate_certs')
     port = host_options.get('http_port')
     version = host_options.get('ontapi')
-    if has_feature(module, 'trace_apis'):
+    trace = has_feature(module, 'trace_apis')
+    if trace:
         logging.basicConfig(filename=LOG_FILE, level=logging.DEBUG)
-        trace = True
-    else:
-        trace = False
-    if has_feature(module, 'always_wrap_zapi'):
-        wrap_zapi = True
+    wrap_zapi |= has_feature(module, 'always_wrap_zapi')
     auth_method = set_auth_method(module, username, password, cert_filepath, key_filepath)
 
-    if HAS_NETAPP_LIB:
-        # set up zapi
-        if auth_method in ('single_cert', 'cert_key'):
-            # override NaServer in netapp-lib to enable certificate authentication
-            server = OntapZAPICx(hostname, module=module, username=username, password=password,
-                                 validate_certs=validate_certs, cert_filepath=cert_filepath,
-                                 key_filepath=key_filepath, style=zapi.NaServer.STYLE_CERTIFICATE,
-                                 auth_method=auth_method, trace=trace)
-            # SSL certificate authentication requires SSL
-            https = True
-        elif auth_method == 'speedy_basic_auth' or wrap_zapi:
-            # override NaServer in netapp-lib to add Authorization header preemptively
-            # use wrapper to handle parse error (mostly for na_ontap_command)
-            server = OntapZAPICx(hostname, module=module, username=username, password=password,
-                                 validate_certs=validate_certs, auth_method=auth_method, trace=trace)
-        else:
-            # legacy netapp-lib
-            server = zapi.NaServer(hostname, username=username, password=password, trace=trace)
-        if vserver:
-            server.set_vserver(vserver)
-        if version:
-            minor = version
-        else:
-            minor = 110
-        server.set_api_version(major=1, minor=minor)
-        # default is HTTP
-        if https:
-            if port is None:
-                port = 443
-            transport_type = 'HTTPS'
-            # HACK to bypass certificate verification
-            if validate_certs is False:
-                if not os.environ.get('PYTHONHTTPSVERIFY', '') and getattr(ssl, '_create_unverified_context', None):
-                    ssl._create_default_https_context = ssl._create_unverified_context
-        else:
-            if port is None:
-                port = 80
-            transport_type = 'HTTP'
-        server.set_transport_type(transport_type)
-        server.set_port(port)
-        server.set_server_type('FILER')
-        return server
-    else:
+    if not HAS_NETAPP_LIB:
         module.fail_json(msg=netapp_lib_is_required())
 
+    # set up zapi
+    if auth_method in ('single_cert', 'cert_key'):
+        # override NaServer in netapp-lib to enable certificate authentication
+        server = OntapZAPICx(hostname, module=module, username=username, password=password,
+                             validate_certs=validate_certs, cert_filepath=cert_filepath,
+                             key_filepath=key_filepath, style=zapi.NaServer.STYLE_CERTIFICATE,
+                             auth_method=auth_method, trace=trace)
+        # SSL certificate authentication requires SSL
+        https = True
+    elif auth_method == 'speedy_basic_auth' or wrap_zapi:
+        # override NaServer in netapp-lib to add Authorization header preemptively
+        # use wrapper to handle parse error (mostly for na_ontap_command)
+        server = OntapZAPICx(hostname, module=module, username=username, password=password,
+                             validate_certs=validate_certs, auth_method=auth_method, trace=trace)
+    else:
+        # legacy netapp-lib
+        server = zapi.NaServer(hostname, username=username, password=password, trace=trace)
+    if vserver:
+        server.set_vserver(vserver)
+    set_zapi_port_and_transport(server, https, port, validate_certs)
+    server.set_api_version(major=1, minor=(version or 110))
+    server.set_server_type('FILER')
+    return server
+
 
 def is_zapi_connection_error(message):
     ''' return True if it is a connection issue '''
     # netapp-lib message may contain a tuple or a str!
-    if isinstance(message, tuple) and isinstance(message[0], ConnectionError):
-        return True
-    if isinstance(message, str) and message.startswith(('URLError', 'Unauthorized')):
-        return True
-    return False
+    try:
+        if isinstance(message, tuple) and isinstance(message[0], ConnectionError):
+            return True
+    except NameError:
+        # python 2.7 does not know about ConnectionError
+        pass
+    return isinstance(message, str) and message.startswith(('URLError', 'Unauthorized'))
 
 
 def is_zapi_write_access_error(message):
-    ''' return True if it is a connection issue '''
+    ''' return True if it is a write access error '''
     # netapp-lib message may contain a tuple or a str!
     if isinstance(message, str) and message.startswith('Insufficient privileges:'):
         return 'does not have write access' in message
     return False
 
 
+def is_zapi_missing_vserver_error(message):
+    ''' return True if it is a missing vserver error '''
+    # netapp-lib message may contain a tuple or a str!
+    return isinstance(message, str) and message == 'Vserver API missing vserver parameter.'
+
+
 def ems_log_event_cserver(source, server, module):
     results = get_cserver(server)
     cserver = setup_na_ontap_zapi(module=module, vserver=results)
@@ -360,7 +366,10 @@ def ems_log_event(source, server, name="Ansible", ident="12345", version=COLLECT
         # Do not fail if we can't connect to the server.
         # The module will report a better error when trying to get some data from ONTAP.
         # Do not fail if we don't have write privileges.
-        if not is_zapi_connection_error(exc.message) and not is_zapi_write_access_error(exc.message):
+        # Do not fail if there is no cserver, as on FSx.
+        if not is_zapi_connection_error(exc.message) \
+           and not is_zapi_write_access_error(exc.message) \
+           and not is_zapi_missing_vserver_error(exc.message):
             # raise on other errors, as it may be a bug in calling the ZAPI
             raise exc
 
@@ -461,20 +470,23 @@ if HAS_NETAPP_LIB:
             try:
                 context = ssl.create_default_context()
             except AttributeError as exc:
-                msg = 'SSL certificate authentication requires python 2.7 or later.'
-                msg += '  More info: %s' % repr(exc)
-                self.module.fail_json(msg=msg)
+                self._fail_with_exc_info('SSL certificate authentication requires python 2.7 or later.', exc)
+
             if not self.validate_certs:
                 context.check_hostname = False
                 context.verify_mode = ssl.CERT_NONE
             try:
                 context.load_cert_chain(self.cert_filepath, keyfile=self.key_filepath)
-            except IOError as exc:      # python 2.7 does not have FileNotFoundError
-                msg = 'Cannot load SSL certificate, check files exist.'
-                msg += '  More info: %s' % repr(exc)
-                self.module.fail_json(msg=msg)
+            except IOError as exc:
+                self._fail_with_exc_info('Cannot load SSL certificate, check files exist.', exc)
+
             return zapi.urllib.request.HTTPSHandler(context=context)
 
+        def _fail_with_exc_info(self, arg0, exc):
+            msg = arg0
+            msg += '  More info: %s' % repr(exc)
+            self.module.fail_json(msg=msg)
+
         def _parse_response(self, response):
             ''' handling XML parsing exception '''
             try:
@@ -562,17 +574,18 @@ if HAS_NETAPP_LIB:
 
 class OntapRestAPI(object):
     ''' wrapper to send requests to ONTAP REST APIs '''
-    def __init__(self, module, timeout=60):
+    def __init__(self, module, timeout=60, host_options=None):
+        self.host_options = module.params if host_options is None else host_options
         self.module = module
-        self.username = self.module.params['username']
-        self.password = self.module.params['password']
-        self.hostname = self.module.params['hostname']
-        self.use_rest = self.module.params['use_rest'].lower()
-        self.cert_filepath = self.module.params['cert_filepath']
-        self.key_filepath = self.module.params['key_filepath']
-        self.verify = self.module.params['validate_certs']
+        self.username = self.host_options['username']
+        self.password = self.host_options['password']
+        self.hostname = self.host_options['hostname']
+        self.use_rest = self.host_options['use_rest'].lower()
+        self.cert_filepath = self.host_options['cert_filepath']
+        self.key_filepath = self.host_options['key_filepath']
+        self.verify = self.host_options['validate_certs']
         self.timeout = timeout
-        port = self.module.params['http_port']
+        port = self.host_options['http_port']
         if port is None:
             self.url = 'https://' + self.hostname + '/api/'
         else:
@@ -585,8 +598,8 @@ class OntapRestAPI(object):
             minor=-1,
             valid=False
         )
-        self.errors = list()
-        self.debug_logs = list()
+        self.errors = []
+        self.debug_logs = []
         self.auth_method = set_auth_method(self.module, self.username, self.password, self.cert_filepath, self.key_filepath)
         self.check_required_library()
         if has_feature(module, 'trace_apis'):
@@ -606,13 +619,10 @@ class OntapRestAPI(object):
             suffix += " - ONTAP version: %s.%s.%s" % current_version
         if use_rest is not None:
             suffix += " - using %s" % ('REST' if use_rest else 'ZAPI')
-        if isinstance(options, list):
-            if len(options) > 1:
-                tag = "any of %s" % options
-            elif len(options) == 1:
-                tag = str(options[0])
-            else:
-                tag = str(options)
+        if isinstance(options, list) and len(options) > 1:
+            tag = "any of %s" % options
+        elif isinstance(options, list) and len(options) == 1:
+            tag = str(options[0])
         else:
             tag = str(options)
         return 'using %s requires ONTAP %s or later and REST must be enabled%s.' % (tag, version, suffix)
@@ -622,7 +632,7 @@ class OntapRestAPI(object):
 
     def fail_if_not_rest_minimum_version(self, module_name, minimum_generation, minimum_major, minimum_minor=0):
         status_code = self.get_ontap_version_using_rest()
-        msgs = list()
+        msgs = []
         if self.use_rest == 'never':
             msgs.append('Error: REST is required for this module, found: "use_rest: %s"' % self.module.params.get('use_rest'))
         if self.is_rest_error:
@@ -642,8 +652,7 @@ class OntapRestAPI(object):
             self.module.fail_json(msg=missing_required_lib('requests'))
 
     def build_headers(self, accept=None, vserver_name=None, vserver_uuid=None):
-        headers = dict()
-        headers['X-Dot-Client-App'] = CLIENT_APP_VERSION % self.module._name
+        headers = {'X-Dot-Client-App': CLIENT_APP_VERSION % self.module._name}
         # accept is used to turn on/off HAL linking
         if accept is not None:
             headers['accept'] = accept
@@ -654,8 +663,7 @@ class OntapRestAPI(object):
             headers['X-Dot-SVM-UUID'] = vserver_uuid
         return headers
 
-    def send_request(self, method, api, params, json=None, accept=None,
-                     vserver_name=None, vserver_uuid=None):
+    def send_request(self, method, api, params, json=None, headers=None):
         ''' send http request and process reponse, including error conditions '''
         url = self.url + api
         status_code = None
@@ -663,7 +671,8 @@ class OntapRestAPI(object):
         json_dict = None
         json_error = None
         error_details = None
-        headers = self.build_headers(accept, vserver_name, vserver_uuid)
+        if headers is None:
+            headers = self.build_headers()
 
         def check_contents(response):
             '''json() may fail on an empty value, but it's OK if no response is expected.
@@ -706,10 +715,14 @@ class OntapRestAPI(object):
             response.raise_for_status()
             json_dict, json_error = get_json(response)
         except requests.exceptions.HTTPError as err:
-            __, json_error = get_json(response)
+            try:
+                __, json_error = get_json(response)
+            except ValueError:
+                json_error = None
             if json_error is None:
                 self.log_error(status_code, 'HTTP error: %s' % err)
                 error_details = str(err)
+
             # If an error was reported in the json payload, it is handled below
         except requests.exceptions.ConnectionError as err:
             self.log_error(status_code, 'Connection error: %s' % err)
@@ -747,7 +760,7 @@ class OntapRestAPI(object):
         runtime = 0
         retries = 0
         max_retries = 3
-        while keep_running:
+        while True:
             # Will run every every <increment> seconds for <timeout> seconds
             job_json, job_error = self.get(url, None)
             job_state = job_json.get('state', None) if job_json else None
@@ -781,47 +794,45 @@ class OntapRestAPI(object):
                     # if the job has failed, return message as error
                     return None, message
                 if job_state not in ('queued', 'running', None):
-                    keep_running = False
                     if error is None:
                         error = job_error
-                else:
+                    break
+                elif runtime >= timeout:
                     # Would like to post a message to user (not sure how)
-                    if runtime >= timeout:
-                        keep_running = False
-                        if job_state != 'success':
-                            self.log_error(0, 'Timeout error: Process still running')
-                            if error is None:
-                                error = 'Timeout error: Process still running'
-                                if job_error is not None:
-                                    error += ' - %s' % job_error
-            if keep_running:
-                time.sleep(increment)
-                runtime += increment
+                    if job_state != 'success':
+                        self.log_error(0, 'Timeout error: Process still running')
+                        if error is None:
+                            error = 'Timeout error: Process still running'
+                            if job_error is not None:
+                                error += ' - %s' % job_error
+                    break
+            time.sleep(increment)
+            runtime += increment
         return message, error
 
-    def get(self, api, params=None):
+    def get(self, api, params=None, headers=None):
         method = 'GET'
-        dummy, message, error = self.send_request(method, api, params)
+        dummy, message, error = self.send_request(method, api, params, headers=headers)
         return message, error
 
-    def post(self, api, body, params=None):
+    def post(self, api, body, params=None, headers=None):
         method = 'POST'
-        dummy, message, error = self.send_request(method, api, params, json=body)
+        dummy, message, error = self.send_request(method, api, params, json=body, headers=headers)
         return message, error
 
-    def patch(self, api, body, params=None):
+    def patch(self, api, body, params=None, headers=None):
         method = 'PATCH'
-        dummy, message, error = self.send_request(method, api, params, json=body)
+        dummy, message, error = self.send_request(method, api, params, json=body, headers=headers)
         return message, error
 
-    def delete(self, api, body=None, params=None):
+    def delete(self, api, body=None, params=None, headers=None):
         method = 'DELETE'
-        dummy, message, error = self.send_request(method, api, params, json=body)
+        dummy, message, error = self.send_request(method, api, params, json=body, headers=headers)
         return message, error
 
-    def options(self, api, params=None):
+    def options(self, api, params=None, headers=None):
         method = 'OPTIONS'
-        dummy, message, error = self.send_request(method, api, params)
+        dummy, message, error = self.send_request(method, api, params, headers=headers)
         return message, error
 
     def set_version(self, message):
@@ -847,6 +858,17 @@ class OntapRestAPI(object):
             return self.ontap_version['generation'], self.ontap_version['major'], self.ontap_version['minor']
         return -1, -1, -1
 
+    def get_node_version_using_rest(self):
+        # using GET rather than HEAD because the error messages are different,
+        # and we need the version as some REST options are not available in earlier versions
+        method = 'GET'
+        api = 'cluster/nodes'
+        params = {'fields': ['version']}
+        status_code, message, error = self.send_request(method, api, params=params)
+        if message and 'records' in message and len(message['records']) > 0:
+            message = message['records'][0]
+        return status_code, message, error
+
     def get_ontap_version_using_rest(self):
         # using GET rather than HEAD because the error messages are different,
         # and we need the version as some REST options are not available in earlier versions
@@ -854,6 +876,12 @@ class OntapRestAPI(object):
         api = 'cluster'
         params = {'fields': ['version']}
         status_code, message, error = self.send_request(method, api, params=params)
+        try:
+            if error and 'are available in precluster.' in error.get('message', ''):
+                # in precluster mode, version is not available :(
+                status_code, message, error = self.get_node_version_using_rest()
+        except AttributeError:
+            pass
         self.set_version(message)
         self.is_rest_error = str(error) if error else None
         if error:
@@ -913,10 +941,7 @@ class OntapRestAPI(object):
         '''
         if filepath is None:
             filepath = '/tmp/ontap_log'
-        if append:
-            mode = 'a'
-        else:
-            mode = 'w'
+        mode = 'a' if append else 'w'
         with open(filepath, mode) as afile:
             if data is not None:
                 afile.write("%s: %s\n" % (str(tag), str(data)))
